@@ -63,7 +63,9 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 	struct host *this, *hosts = NULL;
 	unsigned int mount_default_proto, vers;
 	char *nfsoptions = NULL;
-	unsigned int random_selection = ap->flags & MOUNT_FLAG_RANDOM_SELECT;
+	unsigned int flags = ap->flags &
+			(MOUNT_FLAG_RANDOM_SELECT | MOUNT_FLAG_USE_WEIGHT_ONLY);
+	int nobind = ap->flags & MOUNT_FLAG_NOBIND;
 	int len, status, err, existed = 1;
 	int nosymlink = 0;
 	int ro = 0;            /* Set if mount bind should be read-only */
@@ -112,9 +114,15 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 			while (*comma == ' ' || *comma == '\t')
 				end--;
 
-			if (strncmp("nosymlink", cp, end - cp + 1) == 0)
+			if (strncmp("nosymlink", cp, end - cp + 1) == 0) {
 				nosymlink = 1;
-			else {
+			} else if (strncmp("nobind", cp, end - cp + 1) == 0) {
+				nobind = 1;
+			} else if (strncmp("no-use-weight-only", cp, end - cp + 1) == 0) {
+				flags &= ~MOUNT_FLAG_USE_WEIGHT_ONLY;
+			} else if (strncmp("use-weight-only", cp, end - cp + 1) == 0) {
+				flags |= MOUNT_FLAG_USE_WEIGHT_ONLY;
+			} else {
 				/* Check for options that also make sense
 				   with bind mounts */
 				if (strncmp("ro", cp, end - cp + 1) == 0)
@@ -125,9 +133,9 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 			}
 		}
 
-		debug(ap->logopt, 
-		      MODPREFIX "nfs options=\"%s\", nosymlink=%d, ro=%d",
-		      nfsoptions, nosymlink, ro);
+		debug(ap->logopt, MODPREFIX
+		      "nfs options=\"%s\", nobind=%d, nosymlink=%d, ro=%d",
+		      nfsoptions, nobind, nosymlink, ro);
 	}
 
 	mount_default_proto = defaults_get_mount_nfs_default_proto();
@@ -137,11 +145,11 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 	else if (mount_default_proto == 4)
 		vers = vers | NFS4_VERS_MASK;
 
-	if (!parse_location(ap->logopt, &hosts, what)) {
+	if (!parse_location(ap->logopt, &hosts, what, flags)) {
 		info(ap->logopt, MODPREFIX "no hosts available");
 		return 1;
 	}
-	prune_host_list(ap->logopt, &hosts, vers, nfsoptions, random_selection);
+	prune_host_list(ap->logopt, &hosts, vers, nfsoptions);
 
 	if (!hosts) {
 		info(ap->logopt, MODPREFIX "no hosts available");
@@ -193,7 +201,8 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 			port_opt = strstr(nfsoptions, "port=");
 
 		/* Port option specified, don't try to bind */
-		if (!nosymlink && !port_opt && this->proximity == PROXIMITY_LOCAL) {
+		if (!(nosymlink || nobind) &&
+		    !port_opt && this->proximity == PROXIMITY_LOCAL) {
 			/* Local host -- do a "bind" */
 			const char *bind_options = ro ? "ro" : "";
 
@@ -224,13 +233,32 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 
 		/* Not a local host - do an NFS mount */
 
-		loc = malloc(strlen(this->name) + 1 + strlen(this->path) + 1);
-		if (!loc) {
-			char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
-			error(ap->logopt, "malloc: %s", estr);
-			return 1;
+		if (this->rr && this->addr) {
+			socklen_t len = INET6_ADDRSTRLEN;
+			char n_buf[len + 1];
+			const char *n_addr;
+			n_addr = get_addr_string(this->addr, n_buf, len);
+			loc = malloc(strlen(n_addr) + strlen(this->path) + 4);
+			if (!loc) {
+				char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
+				error(ap->logopt, "malloc: %s", estr);
+				goto forced_fail;
+			}
+			if (this->addr->sa_family == AF_INET6) {
+				strcpy(loc, "[");
+				strcat(loc, n_addr);
+				strcat(loc, "]");
+			} else
+				strcpy(loc, n_addr);
+		} else {
+			loc = malloc(strlen(this->name) + strlen(this->path) + 2);
+			if (!loc) {
+				char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
+				error(ap->logopt, "malloc: %s", estr);
+				goto forced_fail;
+			}
+			strcpy(loc, this->name);
 		}
-		strcpy(loc, this->name);
 		strcat(loc, ":");
 		strcat(loc, this->path);
 
@@ -251,7 +279,7 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 		}
 
 		if (!err) {
-			info(ap->logopt, MODPREFIX "mounted %s on %s", loc, fullpath);
+			debug(ap->logopt, MODPREFIX "mounted %s on %s", loc, fullpath);
 			free(loc);
 			free_host_list(&hosts);
 			return 0;
